// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "PlayerController.hpp"
#include "game/DawnGame.hpp"

using namespace Dawn;

PlayerController::PlayerController(SceneItem *item) : SceneItemComponent(item) {

}

std::vector<SceneItemComponent*> PlayerController::getDependencies() {
  return {
    (this->characterController = item->getComponent<CharacterController2D>())
  };
}

void PlayerController::onStart() {
  assertNotNull(this->characterController);

  useEvent([&](float_t delta){
    // Movement
    auto inputMove = getGame()->inputManager.getAxis2D(
      INPUT_BIND_NEGATIVE_X, INPUT_BIND_POSITIVE_X,
      INPUT_BIND_NEGATIVE_Y, INPUT_BIND_POSITIVE_Y
    ) * delta * moveSpeed;
    
    if(inputMove.x != 0 || inputMove.y != 0) {
      float_t angle = atan2(inputMove.y, inputMove.x);
      glm::vec2 movement(cos(angle), sin(angle));

      transform->setLocalRotation(glm::quat(glm::vec3(0, -angle + 1.5708f, 0)));

      characterController->velocity += movement * delta * moveSpeed;
    }
  }, getScene()->eventSceneUpdate);
}